unit VectorMatrix;
interface
uses
  SysUtils;

type
  Point = Integer; // or Double;
  Vector = Array of Point;
  Matrix = Array of Array of Point;

type
  EVectorSizeMismatch = class(Exception)
  end;
  EMatrixSizeMismatch = class(Exception);


type
  TMatrix = record
  private
    FData: Matrix;
  public
    // conversion
    class operator Explicit(const X: TMatrix): String;

    // unary operators
    class operator Negative(const X: TMatrix): TMatrix;

    // comparison operators
    class operator Equal(const X,Y: TMatrix): Boolean;
    class operator NotEqual(const X,Y: TMatrix): Boolean;

    // binary operators
    class operator Add(const X,Y: TMatrix): TMatrix;
    class operator Subtract(const X,Y: TMatrix): TMatrix;
    class operator Multiply(Factor: Point; const X: TMatrix): TMatrix;
    class operator Multiply(const X: TMatrix; Factor: Point): TMatrix;
    class operator Multiply(const X,Y: TMatrix): TMatrix;
  end;

type
  TVector = record
  private
    FData: Vector;
  public
    // conversion
//  class operator Implicit(const X: Array of Point): TVector;
    class operator Implicit(const X: Vector): TVector;
    class operator Explicit(const X: TVector): String;

    // unary operators
    class operator Negative(const X: TVector): TVector;

    // comparison operators
    class operator Equal(const X,Y: TVector): Boolean;
    class operator NotEqual(const X,Y: TVector): Boolean;

    // binary operators
    class operator Add(const X,Y: TVector): TVector;
    class operator Subtract(const X,Y: TVector): TVector;
    class operator Multiply(Factor: Point; const X: TVector): TVector;
    class operator Multiply(const X: TVector; Factor: Point): TVector;
    class operator Multiply(const X,Y: TVector): TMatrix;
  end;

implementation

{ TVector

class operator TVector.Implicit(const X: array of Point): TVector;
var
  i: Integer;
begin
  SetLength(Result.FData,High(X)+1);
  for i:=0 to High(X) do Result.FData[i] := X[i]
end;
}
class operator TVector.Implicit(const X: Vector): TVector;
var
  i: Integer;
begin
  SetLength(Result.FData,High(X)+1);
  for i:=0 to High(X) do Result.FData[i] := X[i]
end;

class operator TVector.Explicit(const X: TVector): String;
var
  P: Point;
begin
  Result := '(';
  for P in X.FData do Result := Result + FloatToStr(P) + ',';
  if Length(Result) > 1 then Result[Length(Result)] := ')'
  else
    Result := Result + ')'
end;

class operator TVector.Negative(const X: TVector): TVector;
var
  i: Integer;
begin
  SetLength(Result.FData, High(X.FData)+1);
  for i:=0 to High(X.FData) do Result.FData[i] := -X.FData[i]
end;

class operator TVector.Equal(const X,Y: TVector): Boolean;
var
  i: Integer;
begin
  if High(X.FData) <> High(Y.FData) then
    raise EVectorSizeMismatch.Create('TVector.Equal');
  Result := True;
  for i:=0 to High(X.FData) do
    Result := Result AND (X.FData[i] = Y.FData[i])
end;

class operator TVector.NotEqual(const X,Y: TVector): Boolean;
begin
  Result := not (X = Y)
end;

class operator TVector.Add(const X,Y: TVector): TVector;
var
  i: Integer;
begin
  if High(X.FData) <> High(Y.FData) then
    raise EVectorSizeMismatch.Create('TVector.Add');
  SetLength(Result.FData,High(X.FData)+1);
  for i:=0 to High(X.FData) do Result.FData[i] := X.FData[i] + Y.FData[i]
end;

class operator TVector.Subtract(const X,Y: TVector): TVector;
begin
  Result := X + (-Y)
end;

class operator TVector.Multiply(Factor: Point; const X: TVector): TVector;
var
  i: Integer;
begin
  SetLength(Result.FData,High(X.FData)+1);
  for i:=0 to High(X.FData) do Result.FData[i] := Factor * X.FData[i]
end;

class operator TVector.Multiply(const X: TVector; Factor: Point): TVector;
begin
  Result := Factor * X
end;

class operator TVector.Multiply(const X,Y: TVector): TMatrix;
var
  row,col: Integer;
begin
  SetLength(Result.FData, High(X.FData)+1, High(Y.FData)+1);
  for row:=0 to High(X.FData) do
  begin
    for col:=0 to High(Y.FData) do
      Result.FData[row][col] := X.FData[row] * Y.FData[col]
  end
end;

{ TMatrix }

class operator TMatrix.Explicit(const X: TMatrix): String;
var
  row: Integer;
begin
  Result := '(';
  for row:=0 to High(X.FData) do
    Result := Result + String(TVector(Vector(X.FData[row]))) + ',';
  if Length(Result) > 1 then Result[Length(Result)] := ')'
  else
    Result := Result + ')'
end;

class operator TMatrix.Negative(const X: TMatrix): TMatrix;
var
  row,col: Integer;
begin
  SetLength(Result.FData,High(X.FData)+1);
  for row:=0 to High(X.FData) do
  begin
    SetLength(Result.FData[row], High(X.FData[row])+1);
    for col:=0 to High(X.FData[row]) do
      Result.FData[row][col] := - X.FData[row][col]
  end
end;

class operator TMatrix.Equal(const X,Y: TMatrix): Boolean;
var
  row,col: Integer;
begin
  if High(X.FData) <> High(Y.FData) then
    raise EMatrixSizeMismatch.Create('TMatrix.Equal');
  for row:=0 to High(X.FData) do
    if High(X.FData[row]) <> High(Y.FData[row]) then
      raise EMatrixSizeMismatch.Create('TMatrix.Equal');
  Result := True;
  for row:=0 to High(X.FData) do
    for col:=0 to High(X.FData[row]) do
      Result := Result AND (X.FData[row][col] = Y.FData[row][col])
end;

class operator TMatrix.NotEqual(const X,Y: TMatrix): Boolean;
begin
  Result := not (X = Y)
end;

class operator TMatrix.Add(const X,Y: TMatrix): TMatrix;
var
  row,col: Integer;
begin
  if High(X.FData) <> High(Y.FData) then
    raise EMatrixSizeMismatch.Create('TMatrix.Add');
  for row:=0 to High(X.FData) do
    if High(X.FData[row]) <> High(Y.FData[row]) then
      raise EMatrixSizeMismatch.Create('TMatrix.Add');
  SetLength(Result.FData,High(X.FData)+1);
  for row:=0 to High(X.FData) do
  begin
    SetLength(Result.FData[row], High(X.FData[row])+1);
    for col:=0 to High(X.FData[row]) do
      Result.FData[row][col] := X.FData[row][col] + Y.FData[row][col]
  end
end;

class operator TMatrix.Subtract(const X,Y: TMatrix): TMatrix;
begin
  Result := X + (-Y)
end;

class operator TMatrix.Multiply(Factor: Point; const X: TMatrix): TMatrix;
var
  row,col: Integer;
begin
  SetLength(Result.FData, High(X.FData)+1, High(X.FData[0])+1);
  for row:=0 to High(X.FData) do
    for col:=0 to High(X.FData[row]) do
      Result.FData[row,col] := Factor * X.FData[row,col]
end;

class operator TMatrix.Multiply(const X: TMatrix; Factor: Point): TMatrix;
begin
  Result := Factor * X
end;

class operator TMatrix.Multiply(const X,Y: TMatrix): TMatrix;
// NxM * MxO = NxO
var
  row,col: Integer;
  i: Integer;
begin
  for row:=0 to High(X.FData) do
    if High(X.FData[row]) <> High(Y.FData) then
      raise EMatrixSizeMismatch.Create('TMatrix.Multiply');
  SetLength(Result.FData, High(X.FData)+1, High(Y.FData[0])+1);
  for row:=0 to High(X.FData) do
  begin
    for col:=0 to High(Y.FData[0]) do
    begin
      Result.FData[row,col] := 0;
      for i:=0 to High(X.FData[row]) do
        Result.FData[row,col] := Result.FData[row,col] +
          X.FData[row,i] * Y.FData[i,col]
    end
  end
end;

end.
